@@ -133,9 +133,13 @@ pub struct Run {
     pub no_timings: bool,
 
     /// Don't show extra output
-    #[clap(long, short, verbatim_doc_comment)]
+    #[clap(long, short, verbatim_doc_comment, env = "MISE_QUIET")]
     pub quiet: bool,
 
+    /// Don't show any output except for errors
+    #[clap(long, short = 'S', verbatim_doc_comment, env = "MISE_SILENT")]
+    pub silent: bool,
+
     #[clap(skip)]
     pub is_linear: bool,
 
@@ -210,12 +214,7 @@ impl Run {
 
         let num_tasks = tasks.all().count();
         self.is_linear = tasks.is_linear();
-        if let Some(task) = tasks.all().next() {
-            self.output = self.output(task)?;
-            if let TaskOutput::Quiet = self.output {
-                self.quiet = true;
-            }
-        }
+        self.output = self.output(None);
 
         let tasks = Mutex::new(tasks);
         let timer = std::time::Instant::now();
@@ -286,13 +285,13 @@ impl Run {
     fn run_task(&self, env: &BTreeMap<String, String>, task: &Task) -> Result<()> {
         let prefix = task.estyled_prefix();
         if SETTINGS.task_skip.contains(&task.name) {
-            if !self.quiet {
+            if !self.quiet(Some(task)) {
                 eprintln!("{prefix} skipping task");
             }
             return Ok(());
         }
         if !self.force && self.sources_are_fresh(task)? {
-            if !self.quiet {
+            if !self.quiet(Some(task)) {
                 eprintln!("{prefix} sources up-to-date, skipping");
             }
             return Ok(());
@@ -362,13 +361,13 @@ impl Run {
         env: &BTreeMap<String, String>,
         prefix: &str,
     ) -> Result<()> {
+        let config = Config::get();
         let script = script.trim_start();
-        let cmd = trunc(
-            &style::ebold(format!("$ {script} {args}", args = args.join(" ")))
-                .bright()
-                .to_string(),
-        );
-        if !self.quiet {
+        let cmd = style::ebold(format!("$ {script} {args}", args = args.join(" ")))
+            .bright()
+            .to_string();
+        let cmd = trunc(&config.redact(cmd)?);
+        if !self.quiet(Some(task)) {
             eprintln!("{prefix} {cmd}");
         }
 
@@ -453,6 +452,7 @@ impl Run {
         env: &BTreeMap<String, String>,
         prefix: &str,
     ) -> Result<()> {
+        let config = Config::get();
         let mut env = env.clone();
         let command = file.to_string_lossy().to_string();
         let args = task.args.iter().cloned().collect_vec();
@@ -466,8 +466,9 @@ impl Run {
         }
 
         let cmd = format!("{} {}", display_path(file), args.join(" "));
-        let cmd = trunc(&style::ebold(format!("$ {cmd}")).bright().to_string());
-        if !self.quiet {
+        let cmd = style::ebold(format!("$ {cmd}")).bright().to_string();
+        let cmd = trunc(&config.redact(cmd)?);
+        if !self.quiet(Some(task)) {
             eprintln!("{prefix} {cmd}");
         }
 
@@ -494,19 +495,27 @@ impl Run {
         env: &BTreeMap<String, String>,
         prefix: &str,
     ) -> Result<()> {
+        let config = Config::get();
         let program = program.to_executable();
+        let redactions = config.redactions()?;
         let mut cmd = CmdLineRunner::new(program.clone())
             .args(args)
             .envs(env)
-            .raw(self.raw(task));
+            .redact(redactions.clone())
+            .raw(self.raw(Some(task)));
         cmd.with_pass_signals();
-        match self.output {
+        match self.output(Some(task)) {
             TaskOutput::Prefix => cmd = cmd.prefix(format!("{prefix} ")),
+            TaskOutput::Silent => {
+                cmd = cmd.stdout(Stdio::null()).stderr(Stdio::null());
+            }
             TaskOutput::Quiet | TaskOutput::Interleave => {
-                cmd = cmd
-                    .stdin(Stdio::inherit())
-                    .stdout(Stdio::inherit())
-                    .stderr(Stdio::inherit())
+                if redactions.is_empty() {
+                    cmd = cmd
+                        .stdin(Stdio::inherit())
+                        .stdout(Stdio::inherit())
+                        .stderr(Stdio::inherit())
+                }
             }
         }
         let dir = self.cwd(task)?;
@@ -526,24 +535,38 @@ impl Run {
         Ok(())
     }
 
-    fn output(&self, task: &Task) -> Result<TaskOutput> {
-        if self.quiet {
-            Ok(TaskOutput::Quiet)
+    fn output(&self, task: Option<&Task>) -> TaskOutput {
+        if self.silent(task) {
+            TaskOutput::Silent
+        } else if self.quiet(task) {
+            TaskOutput::Quiet
         } else if self.prefix {
-            Ok(TaskOutput::Prefix)
+            TaskOutput::Prefix
         } else if self.interleave {
-            Ok(TaskOutput::Interleave)
-        } else if let Some(output) = &SETTINGS.task_output {
-            Ok(output.parse()?)
+            TaskOutput::Interleave
+        } else if let Some(output) = SETTINGS.task_output {
+            output
         } else if self.raw(task) || self.jobs() == 1 || self.is_linear {
-            Ok(TaskOutput::Interleave)
+            TaskOutput::Interleave
         } else {
-            Ok(TaskOutput::Prefix)
+            TaskOutput::Prefix
         }
     }
 
-    fn raw(&self, task: &Task) -> bool {
-        self.raw || task.raw || SETTINGS.raw
+    fn silent(&self, task: Option<&Task>) -> bool {
+        self.silent || SETTINGS.silent || self.output.is_silent() || task.is_some_and(|t| t.silent)
+    }
+
+    fn quiet(&self, task: Option<&Task>) -> bool {
+        self.quiet
+            || SETTINGS.quiet
+            || self.output.is_quiet()
+            || task.is_some_and(|t| t.quiet)
+            || self.silent(task)
+    }
+
+    fn raw(&self, task: Option<&Task>) -> bool {
+        self.raw || SETTINGS.raw || task.is_some_and(|t| t.raw)
     }
 
     fn jobs(&self) -> usize {
@@ -570,12 +593,15 @@ impl Run {
     }
 
     fn sources_are_fresh(&self, task: &Task) -> Result<bool> {
-        if task.sources.is_empty() && task.outputs.is_empty() {
+        let outputs = task.outputs.paths(task);
+        if task.sources.is_empty() && outputs.is_empty() {
             return Ok(false);
         }
         let run = || -> Result<bool> {
-            let sources = self.get_last_modified(&self.cwd(task)?, &task.sources)?;
-            let outputs = self.get_last_modified(&self.cwd(task)?, &task.outputs)?;
+            let mut sources = task.sources.clone();
+            sources.push(task.config_source.to_string_lossy().to_string());
+            let sources = self.get_last_modified(&self.cwd(task)?, &sources)?;
+            let outputs = self.get_last_modified(&self.cwd(task)?, &outputs)?;
             trace!("sources: {sources:?}, outputs: {outputs:?}");
             match (sources, outputs) {
                 (Some(sources), Some(outputs)) => Ok(sources < outputs),
@@ -583,7 +609,7 @@ impl Run {
             }
         };
         Ok(run().unwrap_or_else(|err| {
-            warn!("sources_are_fresh: {err}");
+            warn!("sources_are_fresh: {err:?}");
             false
         }))
     }
@@ -631,7 +657,8 @@ impl Run {
             Ok(Config::get()
                 .project_root
                 .clone()
-                .unwrap_or_else(|| env::current_dir().unwrap()))
+                .or_else(|| dirs::CWD.clone())
+                .unwrap_or_default())
         }
     }
 
@@ -639,12 +666,17 @@ impl Run {
         if task.sources.is_empty() {
             return Ok(());
         }
-        // TODO
+        if task.outputs.is_auto() {
+            for p in task.outputs.paths(task) {
+                debug!("touching auto output file: {p}");
+                file::touch_file(&PathBuf::from(&p))?;
+            }
+        }
         Ok(())
     }
 
     fn timings(&self) -> bool {
-        !self.quiet
+        !self.quiet(None)
             && !self.no_timings
             && SETTINGS
                 .task_timings
@@ -677,17 +709,13 @@ fn is_glob_pattern(path: &str) -> bool {
     path.chars().any(|c| glob_chars.contains(&c))
 }
 
-fn last_modified_path(
-    root: impl AsRef<std::ffi::OsStr>,
-    paths: &[&String],
-) -> Result<Option<SystemTime>> {
+fn last_modified_path(root: &Path, paths: &[&String]) -> Result<Option<SystemTime>> {
     let files = paths.iter().map(|p| {
         let base = Path::new(p);
-
         if base.is_relative() {
-            base.to_path_buf()
-        } else {
             Path::new(&root).join(base)
+        } else {
+            base.to_path_buf()
         }
     });
 
@@ -727,7 +755,11 @@ fn last_modified_file(files: impl IntoIterator<Item = PathBuf>) -> Result<Option
     Ok(files
         .into_iter()
         .unique()
-        .map(|p| p.metadata().map_err(|err| eyre!(err)))
+        .filter(|p| p.exists())
+        .map(|p| {
+            p.metadata()
+                .map_err(|err| eyre!("{}: {}", display_path(p), err))
+        })
         .collect::<Result<Vec<_>>>()?
         .into_iter()
         .map(|m| m.modified().map_err(|err| eyre!(err)))
@@ -758,13 +790,25 @@ static AFTER_LONG_HELP: &str = color_print::cstr!(
 "#
 );
 
-#[derive(Debug, Default, PartialEq, strum::EnumString)]
+#[derive(
+    Debug,
+    Default,
+    Clone,
+    Copy,
+    PartialEq,
+    strum::EnumString,
+    strum::EnumIs,
+    serde::Serialize,
+    serde::Deserialize,
+)]
+#[serde(rename_all = "snake_case")]
 #[strum(serialize_all = "snake_case")]
 pub enum TaskOutput {
     #[default]
     Prefix,
     Interleave,
     Quiet,
+    Silent,
 }
 
 fn trunc(msg: &str) -> String {
@@ -815,6 +859,7 @@ fn prompt_for_task() -> Result<Task> {
     );
     let mut s = Select::new("Tasks")
         .description("Select a tasks to run")
+        .filtering(true)
         .filterable(true);
     for t in tasks.values().filter(|t| !t.hide) {
         s = s.option(DemandOption::new(&t.name).description(&t.description));
