@@ -1,4 +1,4 @@
-use eyre::{ensure, eyre, Context, Result};
+use eyre::{bail, eyre, Context, Result};
 use indexmap::{IndexMap, IndexSet};
 use itertools::Itertools;
 use once_cell::sync::{Lazy, OnceCell};
@@ -33,10 +33,13 @@ pub mod env_directive;
 pub mod settings;
 pub mod tracking;
 
+use crate::cli::self_update::SelfUpdate;
 use crate::hook_env::WatchFilePattern;
 use crate::hooks::Hook;
 use crate::plugins::PluginType;
+use crate::redactions::Redactions;
 use crate::watch_files::WatchFile;
+use crate::wildcard::Wildcard;
 pub use settings::SETTINGS;
 
 type AliasMap = IndexMap<String, Alias>;
@@ -53,7 +56,9 @@ pub struct Config {
     aliases: AliasMap,
     env: OnceCell<EnvResults>,
     env_with_sources: OnceCell<EnvWithSources>,
+    redactions: OnceCell<IndexSet<String>>,
     shorthands: OnceLock<Shorthands>,
+    hooks: OnceCell<Vec<(PathBuf, Hook)>>,
     tasks: OnceCell<BTreeMap<String, Task>>,
     tool_request_set: OnceCell<ToolRequestSet>,
     toolset: OnceCell<Toolset>,
@@ -295,7 +300,7 @@ impl Config {
         let mut system_tasks = None;
         rayon::scope(|s| {
             s.spawn(|_| {
-                file_tasks = Some(self.load_file_tasks_recursively());
+                file_tasks = Some(self.load_local_tasks());
             });
             global_tasks = Some(self.load_global_tasks());
             system_tasks = Some(self.load_system_tasks());
@@ -338,8 +343,7 @@ impl Config {
         let configs = self.configs_at_root(dir);
         let config_tasks = configs
             .par_iter()
-            .flat_map(|cf| cf.tasks())
-            .cloned()
+            .flat_map(|cf| self.load_config_tasks(&Some(*cf), dir))
             .collect::<Vec<_>>();
         let includes = self.task_includes_for_dir(dir);
         let extra_tasks = includes
@@ -385,7 +389,7 @@ impl Config {
         Ok(tasks.into_values().collect())
     }
 
-    fn load_file_tasks_recursively(&self) -> Result<Vec<Task>> {
+    fn load_local_tasks(&self) -> Result<Vec<Task>> {
         let file_tasks = file::all_dirs()?
             .into_iter()
             .filter(|d| {
@@ -409,7 +413,7 @@ impl Config {
         let cf = self.config_files.get(&*env::MISE_GLOBAL_CONFIG_FILE);
         let config_root = cf.and_then(|cf| cf.project_root()).unwrap_or(&*env::HOME);
         Ok(self
-            .load_config_tasks(&cf)
+            .load_config_tasks(&cf.map(|cf| cf.as_ref()), config_root)
             .into_iter()
             .chain(self.load_file_tasks(&cf, config_root))
             .collect())
@@ -422,17 +426,23 @@ impl Config {
             .map(|p| p.to_path_buf())
             .unwrap_or_default();
         Ok(self
-            .load_config_tasks(&cf)
+            .load_config_tasks(&cf.map(|cf| cf.as_ref()), &config_root)
             .into_iter()
             .chain(self.load_file_tasks(&cf, &config_root))
             .collect())
     }
 
-    fn load_config_tasks(&self, cf: &Option<&Box<dyn ConfigFile>>) -> Vec<Task> {
+    fn load_config_tasks(&self, cf: &Option<&dyn ConfigFile>, config_root: &Path) -> Vec<Task> {
         cf.map(|cf| cf.tasks())
             .unwrap_or_default()
             .into_iter()
             .cloned()
+            .map(|mut t| {
+                if let Err(err) = t.render(config_root) {
+                    warn!("rendering task: {err:?}");
+                }
+                t
+            })
             .collect()
     }
 
@@ -521,12 +531,18 @@ impl Config {
         for cf in self.config_files.values() {
             if let Some(min) = cf.min_version() {
                 let cur = &*version::V;
-                ensure!(
-                    cur >= min,
-                    "mise version {} is required, but you are using {}",
-                    style::eyellow(min),
-                    style::eyellow(cur)
-                );
+                if cur < min {
+                    let min = style::eyellow(min);
+                    let cur = style::eyellow(cur);
+                    if SelfUpdate::is_available() {
+                        bail!(
+                            "mise version {min} is required, but you are using {cur}\n\
+                            Run `mise self-update` to update mise",
+                        );
+                    } else {
+                        bail!("mise version {min} is required, but you are using {cur}");
+                    }
+                }
             }
         }
         Ok(())
@@ -557,19 +573,21 @@ impl Config {
         Ok(env_results)
     }
 
-    pub fn hooks(&self) -> Result<Vec<(PathBuf, Hook)>> {
-        self.config_files
-            .values()
-            .map(|cf| Ok((cf.project_root(), cf.hooks()?)))
-            .filter_map_ok(|(root, hooks)| root.map(|r| (r.to_path_buf(), hooks)))
-            .map_ok(|(root, hooks)| {
-                hooks
-                    .into_iter()
-                    .map(|h| (root.clone(), h))
-                    .collect::<Vec<_>>()
-            })
-            .flatten_ok()
-            .collect()
+    pub fn hooks(&self) -> Result<&Vec<(PathBuf, Hook)>> {
+        self.hooks.get_or_try_init(|| {
+            self.config_files
+                .values()
+                .map(|cf| Ok((cf.project_root(), cf.hooks()?)))
+                .filter_map_ok(|(root, hooks)| root.map(|r| (r.to_path_buf(), hooks)))
+                .map_ok(|(root, hooks)| {
+                    hooks
+                        .into_iter()
+                        .map(|h| (root.clone(), h))
+                        .collect::<Vec<_>>()
+                })
+                .flatten_ok()
+                .collect()
+        })
     }
 
     pub fn watch_file_hooks(&self) -> Result<IndexSet<(PathBuf, WatchFile)>> {
@@ -610,6 +628,48 @@ impl Config {
             .chain(SETTINGS.env_files().iter().map(|p| p.as_path().into()))
             .collect())
     }
+
+    pub fn redactions(&self) -> Result<&IndexSet<String>> {
+        self.redactions.get_or_try_init(|| {
+            let mut redactions = Redactions::default();
+            for cf in self.config_files.values() {
+                let r = cf.redactions();
+                if !r.is_empty() {
+                    let mut r = r.clone();
+                    let (tera, ctx) = cf.tera();
+                    r.render(&mut tera.clone(), &ctx)?;
+                    redactions.merge(r);
+                }
+            }
+            if redactions.is_empty() {
+                return Ok(Default::default());
+            }
+
+            let ts = self.get_toolset()?;
+            let env = ts.full_env()?;
+
+            let env_matcher = Wildcard::new(redactions.env.clone());
+            let var_matcher = Wildcard::new(redactions.vars.clone());
+
+            let env_vals = env
+                .into_iter()
+                .filter(|(k, _)| env_matcher.match_any(k))
+                .map(|(_, v)| v);
+            let var_vals = self
+                .vars
+                .iter()
+                .filter(|(k, _)| var_matcher.match_any(k))
+                .map(|(_, v)| v.to_string());
+            Ok(env_vals.chain(var_vals).collect())
+        })
+    }
+
+    pub fn redact(&self, mut input: String) -> Result<String> {
+        for redaction in self.redactions()? {
+            input = input.replace(redaction, "[redacted]");
+        }
+        Ok(input)
+    }
 }
 
 fn get_project_root(config_files: &ConfigMap) -> Option<PathBuf> {
