@@ -1,11 +1,11 @@
 use crate::config::config_file::toml::{deserialize_arr, TomlParser};
 use crate::config::Config;
-use crate::file;
 use crate::task::task_script_parser::{
     has_any_args_defined, replace_template_placeholders_with_args, TaskScriptParser,
 };
 use crate::tera::{get_tera, BASE_CONTEXT};
 use crate::ui::tree::TreeItem;
+use crate::{dirs, file};
 use console::{truncate_str, Color};
 use either::Either;
 use eyre::{eyre, Result};
@@ -25,14 +25,18 @@ use std::{ffi, fmt, path};
 use xx::regex;
 
 mod deps;
+mod task_dep;
 mod task_script_parser;
+pub mod task_sources;
 
 use crate::config::config_file::ConfigFile;
 use crate::file::display_path;
 use crate::ui::style;
 pub use deps::Deps;
+use task_dep::TaskDep;
+use task_sources::TaskOutputs;
 
-#[derive(Debug, Clone, Eq, PartialEq, Deserialize)]
+#[derive(Debug, Clone, Deserialize)]
 pub struct Task {
     #[serde(skip)]
     pub name: String,
@@ -46,10 +50,12 @@ pub struct Task {
     pub cf: Option<Arc<dyn ConfigFile>>,
     #[serde(skip)]
     pub config_root: Option<PathBuf>,
-    #[serde(default)]
-    pub depends: Vec<String>,
-    #[serde(default)]
-    pub wait_for: Vec<String>,
+    #[serde(default, deserialize_with = "deserialize_arr")]
+    pub depends: Vec<TaskDep>,
+    #[serde(default, deserialize_with = "deserialize_arr")]
+    pub depends_post: Vec<TaskDep>,
+    #[serde(default, deserialize_with = "deserialize_arr")]
+    pub wait_for: Vec<TaskDep>,
     #[serde(default)]
     pub env: BTreeMap<String, EitherStringOrIntOrBool>,
     #[serde(default)]
@@ -61,9 +67,13 @@ pub struct Task {
     #[serde(default)]
     pub sources: Vec<String>,
     #[serde(default)]
-    pub outputs: Vec<String>,
+    pub outputs: TaskOutputs,
     #[serde(default)]
     pub shell: Option<String>,
+    #[serde(default)]
+    pub quiet: bool,
+    #[serde(default)]
+    pub silent: bool,
 
     // normal type
     #[serde(default, deserialize_with = "deserialize_arr")]
@@ -118,7 +128,17 @@ impl Debug for EitherStringOrIntOrBool {
 }
 
 impl Task {
+    pub fn new(path: &Path, prefix: &Path, config_root: &Path) -> Result<Task> {
+        Ok(Self {
+            name: name_from_path(prefix, path)?,
+            config_source: path.to_path_buf(),
+            config_root: Some(config_root.to_path_buf()),
+            ..Default::default()
+        })
+    }
+
     pub fn from_path(path: &Path, prefix: &Path, config_root: &Path) -> Result<Task> {
+        let mut task = Task::new(path, prefix, config_root)?;
         let info = file::read_to_string(path)?
             .lines()
             .filter_map(|line| {
@@ -138,49 +158,59 @@ impl Task {
                 map
             });
         let info = toml::Value::Table(info);
-        let mut tera_ctx = BASE_CONTEXT.clone();
-        tera_ctx.insert("config_root", &config_root);
-        let p = TomlParser::new(&info, get_tera(Some(config_root)), tera_ctx);
+        let p = TomlParser::new(&info);
         // trace!("task info: {:#?}", info);
 
-        let task = Task {
-            name: name_from_path(prefix, path)?,
-            config_source: path.to_path_buf(),
-            config_root: Some(config_root.to_path_buf()),
-            hide: !file::is_executable(path) || p.parse_bool("hide").unwrap_or_default(),
-            aliases: p
-                .parse_array("alias")?
-                .or(p.parse_array("aliases")?)
-                .or(p.parse_str("alias")?.map(|s| vec![s]))
-                .or(p.parse_str("aliases")?.map(|s| vec![s]))
-                .unwrap_or_default(),
-            description: p.parse_str("description")?.unwrap_or_default(),
-            sources: p.parse_array("sources")?.unwrap_or_default(),
-            outputs: p.parse_array("outputs")?.unwrap_or_default(),
-            depends: p.parse_array("depends")?.unwrap_or_default(),
-            wait_for: p.parse_array("wait_for")?.unwrap_or_default(),
-            dir: p.parse_str("dir")?,
-            env: p.parse_env("env")?.unwrap_or_default(),
-            file: Some(path.to_path_buf()),
-            shell: p.parse_str("shell")?,
-            ..Default::default()
-        };
+        task.hide = !file::is_executable(path) || p.parse_bool("hide").unwrap_or_default();
+        task.aliases = p
+            .parse_array("alias")
+            .or(p.parse_array("aliases"))
+            .or(p.parse_str("alias").map(|s| vec![s]))
+            .or(p.parse_str("aliases").map(|s| vec![s]))
+            .unwrap_or_default();
+        task.description = p.parse_str("description").unwrap_or_default();
+        task.sources = p.parse_array("sources").unwrap_or_default();
+        task.outputs = info.get("outputs").map(|to| to.into()).unwrap_or_default();
+        task.depends = p.parse_array("depends").unwrap_or_default();
+        task.depends_post = p.parse_array("depends_post").unwrap_or_default();
+        task.wait_for = p.parse_array("wait_for").unwrap_or_default();
+        task.dir = p.parse_str("dir");
+        task.env = p.parse_env("env")?.unwrap_or_default();
+        task.file = Some(path.to_path_buf());
+        task.shell = p.parse_str("shell");
+        task.render(config_root)?;
         Ok(task)
     }
 
-    // pub fn args(&self) -> impl Iterator<Item = String> {
-    //     if let Some(script) = &self.script {
-    //         // TODO: cli_args
-    //         vec!["-c".to_string(), script.to_string()].into_iter()
-    //     } else {
-    //         self.args
-    //             .iter()
-    //             .chain(self.cli_args.iter())
-    //             .cloned()
-    //             .collect_vec()
-    //             .into_iter()
-    //     }
-    // }
+    /// prints the task name without an extension
+    pub fn display_name(&self) -> String {
+        self.name
+            .rsplitn(2, '.')
+            .last()
+            .unwrap_or_default()
+            .to_string()
+    }
+
+    pub fn is_match(&self, pat: &str) -> bool {
+        if self.name == pat || self.aliases.contains(&pat.to_string()) {
+            return true;
+        }
+        let pat = pat.rsplitn(2, '.').last().unwrap_or_default();
+        self.name == pat || self.aliases.contains(&pat.to_string())
+    }
+
+    pub fn task_dir() -> PathBuf {
+        let config = Config::get();
+        let cwd = dirs::CWD.clone().unwrap_or_default();
+        let project_root = config.project_root.clone().unwrap_or(cwd);
+        for dir in config.task_includes_for_dir(&project_root) {
+            if dir.is_dir() && project_root.join(&dir).exists() {
+                return project_root.join(dir);
+            }
+        }
+        project_root.join("mise-tasks")
+    }
+
     pub fn with_args(mut self, args: Vec<String>) -> Self {
         self.args = args;
         self
@@ -198,41 +228,52 @@ impl Task {
         }
     }
 
-    pub fn all_depends<'a>(&self, config: &'a Config) -> Result<Vec<&'a Task>> {
+    pub fn all_depends(&self) -> Result<Vec<Task>> {
+        let config = Config::get();
         let tasks = config.tasks_with_aliases()?;
-        let mut depends: Vec<&Task> = self
+        let mut depends: Vec<Task> = self
             .depends
             .iter()
-            .map(|pat| match_tasks(&tasks, pat))
+            .chain(self.depends_post.iter())
+            .map(|td| match_tasks(&tasks, td))
             .flatten_ok()
             .filter_ok(|t| t.name != self.name)
             .collect::<Result<Vec<_>>>()?;
         for dep in depends.clone() {
-            depends.extend(dep.all_depends(config)?);
+            depends.extend(dep.all_depends()?);
         }
         Ok(depends)
     }
 
-    pub fn resolve_depends<'a>(
-        &self,
-        config: &'a Config,
-        tasks_to_run: &[&Task],
-    ) -> Result<Vec<&'a Task>> {
-        let tasks_to_run: HashSet<&Task> = tasks_to_run.iter().copied().collect();
+    pub fn resolve_depends(&self, tasks_to_run: &[Task]) -> Result<(Vec<Task>, Vec<Task>)> {
+        let config = Config::get();
+        let tasks_to_run: HashSet<&Task> = tasks_to_run.iter().collect();
         let tasks = config.tasks_with_aliases()?;
-        self.wait_for
+        let depends = self
+            .depends
             .iter()
-            .map(|pat| match_tasks(&tasks, pat))
+            .map(|td| match_tasks(&tasks, td))
+            .flatten_ok()
+            .collect_vec();
+        let wait_for = self
+            .wait_for
+            .iter()
+            .map(|td| match_tasks(&tasks, td))
+            .flatten_ok()
+            .filter_ok(|t| tasks_to_run.contains(t))
+            .collect_vec();
+        let depends_post = tasks_to_run
+            .iter()
+            .flat_map(|t| t.depends_post.iter().map(|td| match_tasks(&tasks, td)))
             .flatten_ok()
-            .filter_ok(|t| tasks_to_run.contains(*t))
-            .chain(
-                self.depends
-                    .iter()
-                    .map(|pat| match_tasks(&tasks, pat))
-                    .flatten_ok(),
-            )
             .filter_ok(|t| t.name != self.name)
-            .collect()
+            .collect::<Result<Vec<_>>>()?;
+        let depends = depends
+            .into_iter()
+            .chain(wait_for)
+            .filter_ok(|t| t.name != self.name)
+            .collect::<Result<_>>()?;
+        Ok((depends, depends_post))
     }
 
     pub fn parse_usage_spec(&self, cwd: Option<PathBuf>) -> Result<(usage::Spec, Vec<String>)> {
@@ -258,7 +299,8 @@ impl Task {
             && spec.cmd.before_help_long.is_none()
             && !self.depends.is_empty()
         {
-            spec.cmd.before_help_long = Some(format!("- Depends: {}", self.depends.join(", ")));
+            spec.cmd.before_help_long =
+                Some(format!("- Depends: {}", self.depends.iter().join(", ")));
         }
         spec.cmd.usage = spec.cmd.usage();
         Ok((spec, scripts))
@@ -294,7 +336,9 @@ impl Task {
 
     pub fn render_markdown(&self, dir: &Path) -> Result<String> {
         let (spec, _) = self.parse_usage_spec(Some(dir.to_path_buf()))?;
-        let ctx = usage::docs::markdown::MarkdownRenderer::new(&spec).with_header_level(2);
+        let ctx = usage::docs::markdown::MarkdownRenderer::new(spec)
+            .with_replace_pre_with_code_fences(true)
+            .with_header_level(2);
         Ok(ctx.render_spec()?)
     }
 
@@ -314,13 +358,19 @@ impl Task {
     }
 
     pub fn dir(&self) -> Result<Option<PathBuf>> {
-        let render = |dir| {
-            let mut tera = get_tera(self.config_root.as_deref());
-            let mut ctx = BASE_CONTEXT.clone();
-            if let Some(config_root) = &self.config_root {
-                ctx.insert("config_root", config_root);
-            }
-            let dir = tera.render_str(dir, &ctx)?;
+        let config = Config::get();
+        if let Some(dir) = &self.dir {
+            Ok(Some(PathBuf::from(dir)))
+        } else if let Some(dir) = self
+            .cf(&config)
+            .as_ref()
+            .and_then(|cf| cf.task_config().dir.clone())
+        {
+            let config_root = self.config_root.clone().unwrap_or_default();
+            let mut tera = get_tera(Some(&config_root));
+            let mut tera_ctx = BASE_CONTEXT.clone();
+            tera_ctx.insert("config_root", &config_root);
+            let dir = tera.render_str(&dir, &tera_ctx)?;
             let dir = file::replace_path(&dir);
             if dir.is_absolute() {
                 Ok(Some(dir.to_path_buf()))
@@ -329,16 +379,6 @@ impl Task {
             } else {
                 Ok(Some(dir.clone()))
             }
-        };
-        let config = Config::get();
-        if let Some(dir) = &self.dir {
-            render(dir)
-        } else if let Some(dir) = self
-            .cf(&config)
-            .as_ref()
-            .and_then(|cf| cf.task_config().dir.clone())
-        {
-            render(&dir)
         } else {
             Ok(self.config_root.clone())
         }
@@ -362,6 +402,47 @@ impl Task {
             }
         })
     }
+
+    pub fn render(&mut self, config_root: &Path) -> Result<()> {
+        let mut tera = get_tera(Some(config_root));
+        let mut tera_ctx = BASE_CONTEXT.clone();
+        tera_ctx.insert("config_root", &config_root);
+        for a in &mut self.aliases {
+            *a = tera.render_str(a, &tera_ctx)?;
+        }
+        self.description = tera.render_str(&self.description, &tera_ctx)?;
+        for s in &mut self.sources {
+            *s = tera.render_str(s, &tera_ctx)?;
+        }
+        self.outputs.render(&mut tera, &tera_ctx)?;
+        for d in &mut self.depends {
+            d.task = tera.render_str(&d.task, &tera_ctx)?;
+            for a in &mut d.args {
+                *a = tera.render_str(a, &tera_ctx)?;
+            }
+        }
+        for d in &mut self.depends_post {
+            d.task = tera.render_str(&d.task, &tera_ctx)?;
+            for a in &mut d.args {
+                *a = tera.render_str(a, &tera_ctx)?;
+            }
+        }
+        for d in &mut self.wait_for {
+            d.task = tera.render_str(&d.task, &tera_ctx)?;
+            for a in &mut d.args {
+                *a = tera.render_str(a, &tera_ctx)?;
+            }
+        }
+        for v in self.env.values_mut() {
+            if let EitherStringOrIntOrBool(Either::Left(s)) = v {
+                *s = tera.render_str(s, &tera_ctx)?;
+            }
+        }
+        if let Some(dir) = &mut self.dir {
+            *dir = tera.render_str(dir, &tera_ctx)?;
+        }
+        Ok(())
+    }
 }
 
 fn name_from_path(prefix: impl AsRef<Path>, path: impl AsRef<Path>) -> Result<String> {
@@ -383,10 +464,18 @@ fn name_from_path(prefix: impl AsRef<Path>, path: impl AsRef<Path>) -> Result<St
         .join(":"))
 }
 
-fn match_tasks<'a>(tasks: &BTreeMap<String, &'a Task>, pat: &str) -> Result<Vec<&'a Task>> {
-    let matches = tasks.get_matching(pat)?.into_iter().cloned().collect_vec();
+fn match_tasks(tasks: &BTreeMap<String, &Task>, td: &TaskDep) -> Result<Vec<Task>> {
+    let matches = tasks
+        .get_matching(&td.task)?
+        .into_iter()
+        .map(|t| {
+            let mut t = (*t).clone();
+            t.args = td.args.clone();
+            t
+        })
+        .collect_vec();
     if matches.is_empty() {
-        return Err(eyre!("task not found: {pat}"));
+        return Err(eyre!("task not found: {td}"));
     };
 
     Ok(matches)
@@ -402,18 +491,21 @@ impl Default for Task {
             cf: None,
             config_root: None,
             depends: vec![],
+            depends_post: vec![],
             wait_for: vec![],
             env: BTreeMap::new(),
             dir: None,
             hide: false,
             raw: false,
             sources: vec![],
-            outputs: vec![],
+            outputs: Default::default(),
             shell: None,
+            silent: false,
             run: vec![],
             run_windows: vec![],
             args: vec![],
             file: None,
+            quiet: false,
         }
     }
 }
@@ -443,13 +535,24 @@ impl PartialOrd for Task {
 
 impl Ord for Task {
     fn cmp(&self, other: &Self) -> Ordering {
-        self.name.cmp(&other.name)
+        match self.name.cmp(&other.name) {
+            Ordering::Equal => self.args.cmp(&other.args),
+            o => o,
+        }
     }
 }
 
 impl Hash for Task {
     fn hash<H: Hasher>(&self, state: &mut H) {
         self.name.hash(state);
+        self.args.iter().for_each(|arg| arg.hash(state));
+    }
+}
+
+impl Eq for Task {}
+impl PartialEq for Task {
+    fn eq(&self, other: &Self) -> bool {
+        self.name == other.name && self.args == other.args
     }
 }
 
@@ -460,7 +563,7 @@ impl TreeItem for (&Graph<Task, ()>, NodeIndex) {
         if let Some(w) = self.0.node_weight(self.1) {
             miseprint!("{}", w.name)?;
         }
-        std::io::Result::Ok(())
+        Ok(())
     }
 
     fn children(&self) -> Cow<[Self::Child]> {
